package org.firstinspires.ftc.teamcode.cometbots;

import static org.firstinspires.ftc.teamcode.PedroConstants.MAX_POWER;

import androidx.annotation.NonNull;

import com.acmerobotics.dashboard.telemetry.TelemetryPacket;
import com.acmerobotics.roadrunner.Action;
import com.acmerobotics.roadrunner.SequentialAction;
import com.acmerobotics.roadrunner.SleepAction;
import com.acmerobotics.roadrunner.ftc.Actions;
import com.qualcomm.robotcore.hardware.Gamepad;
import com.qualcomm.robotcore.hardware.HardwareMap;

import org.firstinspires.ftc.robotcore.external.Telemetry;
import org.firstinspires.ftc.teamcode.pedroPathing.follower.Follower;
import org.firstinspires.ftc.teamcode.states.FieldStates;
import org.firstinspires.ftc.teamcode.subsystem.MotorsSubsystem;

public class CometBotAutoDevelopment {

    /*
    Subsystems
     */
    private MotorsSubsystem motors;

    /*
    Controllers
     */
    public Gamepad GP1;
    public Gamepad GP2;
    public Gamepad currentGP1;
    public Gamepad previousGP1;
    public Gamepad currentGP2;
    public Gamepad previousGP2;
    private Telemetry telemetry;

    public FieldStates fieldStates;
    private boolean centricity = false;

    private Follower follower;
    private HardwareMap hardwareMap;

    public CometBotAutoDevelopment(HardwareMap hardwareMap, Telemetry telemetry, Gamepad gp1, Gamepad gp2) {
        this.motors = new MotorsSubsystem(hardwareMap, telemetry);
        this.GP1 = gp1;
        this.GP2 = gp2;
        this.hardwareMap = hardwareMap;
        this.telemetry = telemetry;
        this.currentGP1 = new Gamepad();
        this.currentGP2 = new Gamepad();
        this.previousGP1 = new Gamepad();
        this.previousGP2 = new Gamepad();
        this.fieldStates = new FieldStates();
        this.follower = new Follower(hardwareMap);
    }

    public class ZeroOutPower implements Action {

        @Override
        public boolean run(@NonNull TelemetryPacket telemetryPacket) {
            follower = new Follower(hardwareMap);
            follower.setMaxPower(0);
            System.out.println("Running ZeroOutPower");
            return follower.isBusy();
        }
    }

    public class ReturnToMaxPower implements Action {

        @Override
        public boolean run(@NonNull TelemetryPacket telemetryPacket) {
            follower = new Follower(hardwareMap);
            follower.setMaxPower(MAX_POWER);
            follower.startTeleopDrive();
            System.out.println("Running ReturnToMaxPower");
            return follower.isBusy();
        }
    }

    public Action zeroOutPower() {
        return new ZeroOutPower();
    }

    public Action returnToMaxPower() {
        return new ReturnToMaxPower();
    }

    public void init() {
        this.motors.init();
        this.fieldStates.setFieldLocation(FieldStates.FieldLocation.TRAVELING);
        follower.setMaxPower(MAX_POWER);
        follower.startTeleopDrive();
    }

    public void update() {
        this.previousGP1.copy(currentGP1);
        this.currentGP1.copy(this.GP1);
        this.previousGP2.copy(currentGP2);
        this.currentGP2.copy(this.GP2);

        this.toFixMotorBlockingIssueFirstMethod();
        this.toFixMotorBlockingIssueSecondMethod();
        this.changeCentricity();

        follower.setTeleOpMovementVectors(-this.GP1.left_stick_y, -this.GP1.left_stick_x, -this.GP1.right_stick_x, centricity);
        follower.update();
        this.telemetry.addData("Field State", this.fieldStates.getFieldLocation());
    }

    public void changeCentricity() {
        if (this.currentGP1.left_bumper && !this.previousGP1.left_bumper) {
            this.centricity = !centricity;
            this.follower.breakFollowing();
            this.follower.startTeleopDrive();
        }
    }

    public void toFixMotorBlockingIssueFirstMethod() {
        if (this.currentGP1.cross && !this.previousGP1.cross) {
            fieldStates.setFieldLocation(FieldStates.FieldLocation.BUCKET);
            Actions.runBlocking(new SequentialAction(
                    this.zeroOutPower(),
                    new SleepAction(3),
                    this.returnToMaxPower()
            ));
            fieldStates.setFieldLocation(FieldStates.FieldLocation.TRAVELING);
        }
    }

    public void toFixMotorBlockingIssueSecondMethod() {
        if (this.currentGP1.circle && !this.previousGP1.circle) {
            this.follower.breakFollowing();
            fieldStates.setFieldLocation(FieldStates.FieldLocation.BUCKET);
            Actions.runBlocking(new SequentialAction(
                    new SleepAction(3)
            ));
            fieldStates.setFieldLocation(FieldStates.FieldLocation.TRAVELING);
            this.follower.startTeleopDrive();
        }
    }
}
