package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.PedroConstants.BACK_LEFT_MOTOR;
import static org.firstinspires.ftc.teamcode.PedroConstants.BACK_LEFT_MOTOR_DIRECTION;
import static org.firstinspires.ftc.teamcode.PedroConstants.BACK_RIGHT_MOTOR;
import static org.firstinspires.ftc.teamcode.PedroConstants.BACK_RIGHT_MOTOR_DIRECTION;
import static org.firstinspires.ftc.teamcode.PedroConstants.FRONT_LEFT_MOTOR;
import static org.firstinspires.ftc.teamcode.PedroConstants.FRONT_LEFT_MOTOR_DIRECTION;
import static org.firstinspires.ftc.teamcode.PedroConstants.FRONT_RIGHT_MOTOR;
import static org.firstinspires.ftc.teamcode.PedroConstants.FRONT_RIGHT_MOTOR_DIRECTION;
import static org.firstinspires.ftc.teamcode.PedroConstants.MAX_POWER;

import androidx.annotation.NonNull;

import com.acmerobotics.dashboard.telemetry.TelemetryPacket;
import com.acmerobotics.roadrunner.Action;
import com.qualcomm.robotcore.hardware.DcMotor;
import com.qualcomm.robotcore.hardware.Gamepad;
import com.qualcomm.robotcore.hardware.HardwareMap;

import org.firstinspires.ftc.robotcore.external.Telemetry;

public class MotorsSubsystem {

    public HardwareMap hardwareMap;
    public Telemetry telemetry;

    public DcMotor frontLeftMotor;
    public DcMotor backLeftMotor;
    public DcMotor frontRightMotor;
    public DcMotor backRightMotor;

    public enum TravelState {
        STOPPED, MOVING
    }

    public TravelState travelState;

    public double power;

    public MotorsSubsystem(HardwareMap hardwareMap, Telemetry telemetry) {
        this.hardwareMap = hardwareMap;
        this.telemetry = telemetry;
        this.power = MAX_POWER;
    }

    public MotorsSubsystem(HardwareMap hardwareMap, Telemetry telemetry, double power) {
        this.hardwareMap = hardwareMap;
        this.telemetry = telemetry;
        this.power = power;
    }

    public void init() {
        frontLeftMotor = hardwareMap.get(DcMotor.class, FRONT_LEFT_MOTOR);
        backLeftMotor = hardwareMap.get(DcMotor.class, BACK_LEFT_MOTOR);
        frontRightMotor = hardwareMap.get(DcMotor.class, FRONT_RIGHT_MOTOR);
        backRightMotor = hardwareMap.get(DcMotor.class, BACK_RIGHT_MOTOR);

        frontLeftMotor.setDirection(FRONT_LEFT_MOTOR_DIRECTION);
        backLeftMotor.setDirection(BACK_LEFT_MOTOR_DIRECTION);
        frontRightMotor.setDirection(FRONT_RIGHT_MOTOR_DIRECTION);
        backRightMotor.setDirection(BACK_RIGHT_MOTOR_DIRECTION);

        frontLeftMotor.setZeroPowerBehavior(DcMotor.ZeroPowerBehavior.BRAKE);
        backLeftMotor.setZeroPowerBehavior(DcMotor.ZeroPowerBehavior.BRAKE);
        frontRightMotor.setZeroPowerBehavior(DcMotor.ZeroPowerBehavior.BRAKE);
        backRightMotor.setZeroPowerBehavior(DcMotor.ZeroPowerBehavior.BRAKE);

        this.setState(TravelState.STOPPED);
    }

    public void setFrontLeftMotorPower(double power) {
        frontLeftMotor.setPower(power);
    }

    public void setBackLeftMotorPower(double power) {
        backLeftMotor.setPower(power);
    }

    public void setFrontRightMotorPower(double power) {
        frontRightMotor.setPower(power);
    }

    public void setBackRightMotorPower(double power) {
        backRightMotor.setPower(power);
    }


    public void setState(TravelState travelState) {
        this.travelState = travelState;
    }

    public TravelState getState() {
        return this.travelState;
    }

    public void setPower(DcMotor motor, double power) {
        motor.setPower(power);
        if (power < 0.05) {
            this.setState(TravelState.MOVING);
        } else {
            this.setState(TravelState.STOPPED);
        }
    }
}
